#include <algorithm>

#include "groupscreen.h"
#include "fetch_group.h"
#include "fonts.h"

GroupScreen::GroupScreen(pqxx::connection &conn, unsigned tournament, unsigned round, unsigned parallel)
	: tournament(tournament), round(round), parallel(parallel), scores_changed(conn, "scores"), conn(conn), valid(false)
{
}

GroupScreen::~GroupScreen()
{
}

bool GroupScreen::check_invalidated()
{
	// we might want to do this slightly more sophisticated later, but for now this will do
	return !valid || scores_changed.get_flag();
}

void GroupScreen::draw(unsigned char *buf)
{
	scores_changed.reset_flag();

	Group group;
	conn.perform(FetchGroup(tournament, round, parallel, &group));

	memset(buf, 0, 800 * 600 * 4);

	// main heading
	char heading[64];
	if (parallel == 0) {
		sprintf(heading, "Round %u", round);
	} else {
		sprintf(heading, "Round %u, Group %u", round, parallel);
	}

	{
		unsigned width = my_draw_text(heading, NULL, 48.0);
		my_draw_text(heading, buf, 48.0, 800/2 - width/2, 60);
	}
	
	// find out how wide each column has to be
	unsigned width[16];
	for (unsigned i = 0; i < 16; ++i)
		width[i] = 0;

	unsigned max_num_width = my_draw_text("88888", NULL, 22.0);

	for (std::vector<Player>::const_iterator i = group.players.begin(); i != group.players.end(); ++i) {
		width[0] = std::max(width[0], my_draw_text(i->nick, NULL, 18.0));

		unsigned col = 1;
		for (std::vector<Score>::const_iterator j = i->scores.begin(); j != i->scores.end(); ++j, ++col) {
			if (j->chosen) {
				width[col] = std::max(width[col], my_draw_text(j->song.title, NULL, 12.0) + 
						 		  max_num_width + 10);
			} else {		
				width[col] = std::max(width[col], my_draw_text(j->song.short_title, NULL, 12.0));
				width[col] = std::max(width[col], max_num_width);
			}
		}
	}	

	// make column headings from the first player's songs
	unsigned col = 1, x = 40 + width[0];
	for (std::vector<Score>::const_iterator i = group.players[0].scores.begin(); i != group.players[0].scores.end(); ++i, ++col) {
		if (!i->chosen) {
			unsigned this_width = my_draw_text(i->song.short_title, NULL, 12.0);
			my_draw_text(i->song.short_title, buf, 12.0, x + width[col] / 2 - this_width / 2, 100);
		}
		x += width[col] + 20;
	}
	
	// show all the players and the scores
	unsigned y = 140;
	for (std::vector<Player>::const_iterator i = group.players.begin(); i != group.players.end(); ++i) {
		my_draw_text(i->nick, buf, 18.0, 20, y);

		unsigned x = 40 + width[0];

		unsigned col = 1;
		for (std::vector<Score>::const_iterator j = i->scores.begin(); j != i->scores.end(); ++j, ++col) {
			char text[16];
			sprintf(text, "%u", j->score);
	
			unsigned this_width = my_draw_text(text, NULL, 22.0);
			if (j->chosen) {
				if (j->score != -1) {
					my_draw_text(text, buf, 22.0, x + max_num_width - this_width, y);
				}
				my_draw_text(j->song.title, buf, 12.0, x + max_num_width + 10, y);
			} else {
				if (j->score != -1) {
					my_draw_text(text, buf, 22.0, x + width[col] / 2 - this_width / 2, y);
				}
			}
			x += width[col] + 20;
		}

		y += 40;
	}
	
	valid = true;
}

